#!/bin/sh
#
# Add-ons for user-defined installation
#

printf "\nExecute bspinst_addons.sh\n\n"
#source ./bspinst_api.sh

android_data_init ()
{
	data_mnt=$mnt_data
	/bin/mkdir -p ${data_mnt}

	if [ $ROOT_DEV -eq $DEV_NAND ]; then
		msg "copy data to Android-Data partition(Nand)..."
		mtdblock_data=`get_mtdblock "${blk_Nand_AndroidData}"`
		/bin/mount -t yaffs2 -o rw ${mtdblock_data}  ${data_mnt}
	elif [ $ROOT_DEV -eq $DEV_TF ]; then
		msg "copy data to Android-Data partition(TF)..."
		/bin/mount /dev/mmcblk1p5  ${data_mnt}
	elif [ $ROOT_DEV -eq $DEV_UDISK ]; then
		msg "copy data to Android-Data partition(UDISK)..."
		/bin/mount /dev/sda5  ${data_mnt}
	else
		msg "*E* Unknown root device!"
		return 2
	fi
	
	if [ $? -ne 0 ] ; then
		msg "*E*  Failed to mount Android-Data partition"
	    return 3
	fi

#   extract patch packages to Android data partition
	echo "[WMT] copy data package to data partition..."
	/bin/tar xf $src_dir/packages/fs_patch/root_patch/.restore/data.tar -C  ${data_mnt} >/dev/null 
	ratio 85
	/bin/tar xf $src_dir/packages/fs_patch/root_patch/.restore/data_app.tar -C  ${data_mnt} >/dev/null 
#   copy patch packages to Android data partition
	/bin/cp -aR $src_dir/packages/fs_patch/data_patch/. ${data_mnt}
	/bin/chmod -R 777  ${data_mnt}/app
	ratio 90

#   no need to chmod/chown if the patch is dumped from Anroid runtime system
#   normally, you can't modify this dumped patch
#   you must use sudo to modify and re-compress the pactch if want to modify it
#	/bin/chmod 0660  ${data_mnt}/data/com.android.providers.settings/databases/*
#   1000 is uid:guid of settings.apk's
#	/bin/chown -hR 1000:1000  ${data_mnt}/data/com.android.providers.settings
	sync
}

create_udc ()
{
	#add_missing_files if it is not included in ramdisk
	#tar zxf $src_dir/packages/fs_patch/.restore/mk_utility*.tgz -C / >/dev/null 

	mtd_max=2147483648
	mtd_len=`get_mtd_len $blk_Nand_LocalDisk`
	if [ "$(($mtd_max))" -lt "$(($mtd_len))" ]; then
		mtd_len=$(($mtd_max))
	fi
	mtd_lendec=`echo $(($mtd_len))`
	mtd_full_reserve=$(( $mtd_lendec + 1073741824 ))
	mtd_full_reserve=$(( $mtd_full_reserve * 5 ))
	mtd_full_reserve=$(( $mtd_full_reserve / 100 ))
	mtd_small_reserve=$(( $mtd_lendec * 5 ))
	mtd_small_reserve=$(( $mtd_small_reserve / 100 ))
	mtd_reserve=$(( $mtd_lendec - $mtd_full_reserve ))	
	mtd_min=10485760
	if [ "$(($mtd_reserve))" -le "$(($mtd_min))" ]; then
		mtd_lendec=$(( $mtd_lendec - $mtd_small_reserve ))
	else
		mtd_lendec=$(( $mtd_lendec - $mtd_full_reserve ))
	fi
	    
	echo "[WMT] Creating UDC udc_file...\n"
	dd bs=1 seek=$mtd_lendec count=1 if=/dev/zero of=$1
	mkdosfs $1	
	sync
}

localdisk_init ()
{
	/bin/mkdir -p ${mnt_localdisk}
	if [ $ROOT_DEV -eq $DEV_NAND ]; then
		if [ $INIT_LOCAL_DISK == 1 ]; then
			nand_erase $blk_Nand_LocalDisk
		fi		
		
		mtdblock=`get_mtdblock "LocalDisk"`

		mount -t yaffs2 -o rw $mtdblock  $mnt_localdisk
		if [ $? -ne 0 ] ; then
			msg "*E*  Failed to get localdisk partition!!"
		    return 1
		fi
	elif [ $ROOT_DEV -eq $DEV_TF ] || [ $ROOT_DEV -eq $DEV_UDISK ]; then
		local mtd_local=
		if [ $ROOT_DEV -eq $DEV_TF ]; then
			msg "Prepare localdisk partition(TF)..."
			mtd_local=/dev/mmcblk1p7
		elif [ $ROOT_DEV -eq $DEV_UDISK ]; then
			msg "Prepare localdisk partition(UDISK)..."
			mtd_local=/dev/sda7
		fi

		if [ $INIT_LOCAL_DISK == 1 ]; then
			mkdosfs ${mtd_local}
		fi

		mount -t vfat -o utf8 ${mtd_local} ${mnt_localdisk}
		if [ $? -ne 0 ] ; then
			msg "*E*  Failed to get localdisk partition!!"
		    return 1
		fi
	else
		msg "*E* Unknown root device!"
		return 1
	fi

	if [ $INIT_LOCAL_DISK == 0 ]; then
		msg "Just remove old DB in LocalDisk"
		if [ -d  ${mnt_localdisk}/.database ]; then
			echo "[WMT] removed scanner DB in LocalDisk"
			/bin/rm -rf ${mnt_localdisk}/.database
			sync
		fi
		
		return 0
	fi

#   copy pre-files to LocalDisk partition
	msg "copy pre-files to LocalDisk..."
	/bin/cp -aR $src_dir/packages/localdisk/. ${mnt_localdisk}
	sync
	echo "[WMT] copy pre-files to LocalDisk, done"
	
	return 0
}

if [ $ROOT_DEV -eq $DEV_NAND ]; then
#	nand_erase "${blk_Nand_AndroidData}"
	nand_erase "${blk_Nand_AndroidCache}"
elif [ $ROOT_DEV -eq $DEV_TF ]; then
	msg "Format Android-Data partition ..."
#	umount /dev/mmcblk1p5
#	mkfs.ext3 /dev/mmcblk1p5
#	if [ $? -ne 0 ] ; then
#		msg "*E*  Failed to format Android-Data partition(TF)..."
#	fi

	msg "Format Android-Cache partition ..."
	umount /dev/mmcblk1p6
	mkfs.ext3 /dev/mmcblk1p6
	if [ $? -ne 0 ] ; then
		msg "*E*  Failed to format Android-Cache partition(TF)..."
	fi
elif [ $ROOT_DEV -eq $DEV_UDISK ]; then
	msg "Format Android-Data partition ..."
#	umount /dev/sda5
#	mkfs.ext3 /dev/sda5
#	if [ $? -ne 0 ] ; then
#		msg "*E*  Failed to format Android-Data partition(UDISK)..."
#	fi

	msg "Format Android-Cache partition ..."
	umount /dev/sda6
	mkfs.ext3 /dev/sda6
	if [ $? -ne 0 ] ; then
		msg "*E*  Failed to format Android-Cache partition(UDISK)..."
	fi
fi


#android_data_init

localdisk_init

printf "\nExecute bspinst_addons.sh done. \n\n"

