# bspinst_api.sh
#
# Linux BSP installer drawing API for WonderMedia ARM SoC.
#
# Written by Vincent Chen <vincentchen@wondermedia.com.tw>
#
# Copyright 2008-2009 WonderMedia Corporation. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY WONDERMEDIA CORPORATION ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL WONDERMEDIA CORPORATION OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.


source ${src_dir}/bin/bspinst_var.sh


# WMT_Fdisk - fdisk TF/UDisk
# @patition_cfg in shell script
# @dev_name of TF/UDisk
WMT_Fdisk ()
{
	tar zxf $src_dir/tools/mk_utility*.tgz -C / >/dev/null 
	local partition_cmd=$1
	local dev_name=$2

	local t=5
	while [ $t -gt 0 ]; do
		if [ -e $dev_name ]; then
			break;
		else
			echo "[WMT] waiting for root device($dev_name) ..."
			if [ $t -eq 1 ]; then
				msg "Root device($dev_name) not found, upgrading stopped!"
				exit 0
			else
				sleep 1
				let "t = t - 1"
			fi
		fi
	done
	
	msg "Fdisk root device($dev_name) ..."
	killall udevd
	sleep 5

	if [ $ROOT_DEV -eq $DEV_TF ]; then
		#Jody: is here need 'p' ?
		umount ${dev_name}p*
		${partition_cmd} | fdisk $dev_name

		#Jody: system will not create dev-nod for new MMC partition, we do it
		mknod "${dev_name}p1" b 179 9
		mknod "${dev_name}p2" b 179 10
		mknod "${dev_name}p3" b 179 11
		mknod "${dev_name}p5" b 179 13
		mknod "${dev_name}p6" b 179 14
		mknod "${dev_name}p7" b 179 15
	elif [ $ROOT_DEV -eq $DEV_UDISK ]; then
		umount ${dev_name}*
		${partition_cmd} | fdisk $dev_name
	fi
}

unmount_all ()
{
	cd /
	sync
	if [ $ROOT_DEV -eq $DEV_NAND ]; then
		sync
		/bin/umount /LocalDisk
		/sbin/losetup -d $udc_loop_dev
		sync
	else
		/bin/umount $mnt_kernel
	fi
	
	/bin/umount $mnt_localdisk
	/bin/umount $mnt_data
	/bin/umount $mnt_fs
}

# Quiet - do command in silence
# @command
Quiet ()
{
	$@ 2>/dev/null >/dev/null
#	$@

	return $?
}


# wait_install
#
# Wait a few seconds before installation
wait_install ()
{
	t=$1
	while [ $t -gt 0 ]; do
		msg "Installation will begin in $t seconds"
		sleep 1
		let "t = t - 1"
	done
}

# wait_reboot
#
# Wait a few seconds before reboot
wait_reboot ()
{
	t=$1
	while [ $t -gt 0 ]; do
		msg "System will reboot in $t seconds"
		sleep 1
		let "t = t - 1"
	done
}


# host_arch_detect
#
# Detect ARMv4t or ARMv5t
host_arch_detect ()
{
	echo "armv5t"

	return 0
}

os_type ()
{
	if [ ! -e /proc/binder/state ]; then
		echo "Linux"
	else
		echo "Android"
	fi
}

# rpc_test
#
# Test if RPC service is ready
rpc_test ()
{
	local st=

	st=`rpc timer get_status | cut -d\  -f 3`

	if [ "$st" != "off" ] ; then
		return 127
	fi

	return 0
}

# wait_rpc_ready 
#
# Wait until RPC service is ready
wait_rpc_ready ()
{
	local st=
	while [ "$st" != "off" ] ; do
		st=`rpc timer get_status | cut -d\  -f 3`
	done
}

# regular_name - translate @name to reqular expression.
# @name
regular_name ()
{
	local name=$1
	echo -n $name | sed 's/\//\\\//g' | sed 's/\./\\\./g'
}

# base_name - extract filename from whole path name
# @path_name
# @full_name
base_name ()
{
	local path_name=$1
	local full_name=$2

	local reg_path_name=`regular_name $path_name`
	echo -n $full_name | sed "s/$reg_path_name//g"
}

# fsearch - search a file by pattern match
# @from		the searching directory
# @pattern	the file name pattern
# @result	the matched file name found
fsearch ()
{
	local from=$1
	local pattern=
	local result=

	eval "pattern=\$$2"

	if [ -z "$pattern" ]; then
		return 
	fi

	#printf "> Search for firmware ... "

	for i in `find $from -type f -name "$pattern" -follow | sort -r`
	do
		result=$i
		break
	done
	
	if [ ! -e "$result" ]; then
		#printf "Not found (%s) \n" $pattern
		printf "fsearch: file is not found (%s) \n" $pattern
	else
		#printf "Found \n%s\n" $result
		eval "$2=\"$result\""

		if [ $# == 3 ]; then
			eval "$3=\"$result\""
		fi
	fi
}

# fcheck - terminate program if the file doesn't exist
# @file		the name of the target file
fcheck ()
{
	local file=$1

	if [ ! -e "$file" ]; then
		printf "The required file is missing. ($file)\n"
		exit 0;
	fi
}

# get_var - load environment variables 
# @args
get_var ()
{
	for exp in $@; do
		eval $exp
	done
}

# get_mtd - get mtd device by name
# @name
get_mtd ()
{
	desc=`echo $1 | sed 's/\ /\\\ /g'`
	cat /proc/mtd | grep "\"$desc\"" | cut -d: -f1 | sed 's/mtd/\/dev\/mtd/g'
}

# get_mtdblock - get mtdblock device by name
# @name
get_mtdblock ()
{
	get_mtd "$1" | sed 's/mtd/mtdblock/g'
}

# get_mtd_len - get mtd length by name
# @name
get_mtd_len ()
{
	tmp=`cat /proc/mtd |grep "\"$1\""| cut -d\  -f 2`
	len=`printf "0x%x" 0x$tmp`

	echo $len
}

# sf_inst - copy from a file onto SPI flash
# @name
# @file
#
# The default search path is the current folder
# if $src_dir is invalid.
sf_inst ()
{
	local name=$1
	local file=$2
	local _src_dir=
	local mtd=

	mtd=`get_mtd "$name"`

	if [ -z "$mtd" ]; then
		echo "[WMT] *E* No MTD partition called $name."
		return 0
	fi

	if [ -z "$src_dir" ]; then
		_src_dir="."
	else
		_src_dir="$src_dir"
	fi

	Quiet fsearch $_src_dir file

	if [ -e "$file" ]; then
		printf "Update %s to %s ... " $file $mtd
		Quiet flash_eraseall $mtd
		if [ -e /usr/sbin/flashcp ]; then
			Quiet flashcp $file $mtd
		else
			cat $file > $mtd
		fi
		printf "done\n"
	else
		echo "[WMT] *E* not found $file, skip update $name."
	fi
}

# nand_erase: erase nand partition
# @discription of nand partition
# 
nand_erase ()
{
	local name=$1
	local mtd=

	mtd=`get_mtd "$name"`

	if [ -z "$mtd" ]; then
		echo "No MTD partition called $name."
		return 1
	fi

	if [ ! -e "$mtd" ]; then
		return 2
	fi
	
	echo "flash eraseall $mtd ... "
	Quiet flash_eraseall $mtd
	echo "flash eraseall done"
	return 0
}

# nand_inst - copy from a file onto Nand flash
# @name
# @file
#
# The default search path is the current folder
# if $src_dir is invalid.
nand_inst ()
{
	local name=$1
	local file=$2
	local _src_dir=
	local mtd=`get_mtd "$name"`

	if [ -z "$mtd" ]; then
		echo "No MTD partition called $name."
		return 1
	fi

	if [ -z "$src_dir" ]; then
		_src_dir="."
	else
		_src_dir="$src_dir"
	fi

	Quiet fsearch $_src_dir file

	if [ ! -e "$file" ]; then
		return 2
	fi

	printf "Update %s to %s ... " $file $mtd
	nand_erase $name
	if [ -e /usr/sbin/nandwrite ]; then
		Quiet nandwrite -p $mtd $file
	else
		cat $file > $mtd
	fi
	printf "done\n"
	
	return 0
}

# GetVersion_old - get version from U-Boot, W-Load, and Kernel
# @Keyword
GetVersion_old ()
{
	local ver=

	case $1 in
	U-Boot*)
		f=`get_mtd "u-boot-SF"`
		ver=`strsearch $f "U-Boot Version" 3`
		;;
	W-Load*)
		f=`get_mtd "w-load-SF"`
		ver=`strsearch $f "W-Load Version" 2`
		;;
	Kernel*)
		f="/proc/version"
		ver=`strsearch $f "Kernel" 1`
		;;	
	esac

	if [ "$ver" == "" ]; then
		ver="0.00.00.00"
	fi

	echo $1 $ver
}

# GetVersion - get version from /proc/version
# @Keyword
GetVersion ()
{
	local ret
	if [ -e "$src_dir/version" ]; then
		ret=`cat $src_dir/version | grep $1 | cut -d':' -f 2`
	fi
	if [ "$ret" ]; then
		echo "$ret"
	else
		echo " "
	fi
}

# make_uboot_script
#
# U-boot script autogen. The scripts are used for loadb.
make_uboot_script ()
{
	local _src_dir=

	if [ -z "$src_dir" ]; then
		_src_dir="."
	else
		_src_dir="$src_dir"
	fi

	rm $_src_dir/env/uboot_env_*.bin 2>/dev/null

	src_list=`find $_src_dir -name "uboot_env_*"`

	for i in $src_list; do
		SRC=$i
		DST=${SRC}.img

		$_src_dir/bin/mkimage -A arm -O linux -T script -n "uboot env" -d $SRC $DST
	done
}

mkscriptcmd ()
{
	if [ ! -e "$src_dir/scriptcmd" ] && [ -e "$src_dir/env/scriptcmd.txt" ]; then
		$src_dir/bin/mkimage -A arm -O linux -T script \
			-n "scriptcmd" \
			-d $src_dir/env/scriptcmd.txt $src_dir/scriptcmd
	fi
}


# add_missing_files
#
# Add required files if necessary
add_missing_files ()
{
	local p01="rpcdemo-*.tgz"
	local p02="directfb-1.4*.tgz"
	local p03="linux-fusion-8*.tgz"
	local p04="libwmtGL-*.tgz"
	local p05="fontconfig*.tgz"
	local p06="freetype2*.tgz"
	local p07="mtd-utils*.tgz"
	local p08="sysfsutils*.tgz"
	local p09="zlib*.tgz"

	# add missing files
	for pkg in $p01 $p02 $p03 $p04 $p05 $p06 $p07 $p08 $p09
	do
		fsearch $src_dir pkg
		echo "Add missing $pkg"
		tar zxf $pkg -C /
	done

	# load linux-fusion-8.1.1
	if [ -e "/etc/rc.d/rc0.d/S48fusion" ]; then
		sh /etc/rc.d/rc0.d/S48fusion
	else
		modprobe fusion
	fi
}

hex_to_dec ()
{
	printf "%u" $1
}

dec_to_hex ()
{
	printf "0x%x" $1
}

get_mtd_offset ()
{
	mtdlist=`cat /proc/wmt_mtd|cut -d\  -f2,3|tr \  @|sed 's/"//g'`

	flg_error="error"
	offset=$flg_error
	for i in $mtdlist; do
		mtd=`echo $i|cut -d@ -f2`
		if [ "$mtd" == "$1" ]; then
			offset=`echo $i|cut -d@ -f1`
			break;
		fi
	done
	
	if [ "$offset" == $flg_error ]; then
		echo "[WMT] *E* Failed to get offset of $1"
	else
		echo "0x$offset"
	fi
}


# do_setenv_nand
# @uboot_env	U-boot script in plain text format
# @mtd		MTD device used by U-Boot env.
#
# Save uboot environment variables from u-boot plain text script
#
# Usage:
#     do_setenv_nand /mnt/mmcblk0p1/env/uboot_env_sf /dev/mtd7
do_setenv_nand ()
{
	local mtd=$2
	local flag=$3

	local crc32=${src_dir}/bin/crc32
	local infile=$1
	local outfile=/tmp/uboot_env.raw
	local tmpfile=/tmp/uboot_env.tmp

	echo "[do_setenv_nand] infile=$infile"
	echo "[do_setenv_nand] outfile=$outfile"
	echo "[do_setenv_nand] tmpfile=$tmpfile"

	dd if=/dev/zero of=${outfile} bs=1k count=64

	cat ${infile} | \
	tr '\n' '~' | sed 's/\\~//g' | tr '~' '\n' | \
	sed 's/\ \+/ /g' | \
	grep setenv | sed 's/setenv //' | sed 's/ /=/' | \
	sed 's/\\;/;/g' | tr "\n" "\0" > ${tmpfile}

	dd if=${tmpfile} of=${outfile} bs=1 seek=4 count=65532 conv=notrunc

	$crc32 -s 4 -x 1 ${outfile}
	$crc32 -s 4 -r ${outfile} > ${tmpfile}
	dd if=${tmpfile} of=${outfile} bs=1 seek=0 count=4 conv=notrunc

	flash_eraseall $mtd
	nandwrite -p -s 0  $mtd ${outfile}

	rm ${tmpfile} ${outfile}
}

# setenv_nand
# @uboot_env	U-boot script in plain text format
#
# Save uboot environment variables from u-boot plain text script
#
# Usage:
#     setenv "uboot_env_nand"
setenv_nand ()
{
	local mtd_name=
	local mtd=
	local uboot_env=$1
	local tmpfile=/tmp/setenv_nand.tmp

	rm -f ${tmpfile}
	if [ -e ${src_dir}/env/env_uboot ]; then
		cat ${src_dir}/env/env_uboot >> ${tmpfile}
	fi
	if [ -e ${src_dir}/env/${uboot_env} ]; then
		cat ${src_dir}/env/${uboot_env} >> ${tmpfile}
	fi
	if [ -e ${src_dir}/env/uboot_env_tail ]; then
		cat ${src_dir}/env/uboot_env_tail >> ${tmpfile}
	fi

	mtd_name="u-boot env. cfg. 1-NAND"
	mtd=`get_mtd "${mtd_name}"`

	if [ ! -z "${mtd}" ]; then
		do_setenv_nand ${tmpfile} ${mtd}
	else
		echo "No MTD partition called ${mtd_name}."
	fi

	rm -f ${tmpfile}
}

# generate a random MAC address
# random_mac
# @file_name tmp file name
#
# Generate a random MAC address to tmp file
#
# Usage:
#     random_mac /tmp/uboot_env.tmp
random_mac ()
{
	local outfile=$1
	local name=ethaddr
	local random=`head -200 /dev/urandom | cksum | cut -f1 -d" "`

	local a1=`echo $random | cut -c1-2`
	local a2=`echo $random | cut -c3-4`
	local a3=`echo $random | cut -c5-6`
	
	echo "random_mac: 00:40:63:$a1:$a2:$a3"
	
	echo "setenv $name 00:40:63:$a1:$a2:$a3" >> ${outfile}
}

# restore_env
# @env_name	 uboot environment variable name
# @file_name tmp file name
#
# Save current uboot environment variable to tmp file
#
# Usage:
#     restore_env ethaddr /tmp/uboot_env.tmp
restore_env ()
{
	local name=$1
	local outfile=$2
	
	local value=`${src_dir}/bin/getenv $name`
	local ret=`echo $value | grep failed`
	
	if [ "$value" = "" ] || [ -n "$ret" ]; then
		echo "[WMT] *I* restore serialno failed! "
	else 
		echo "restore: $name=$value to ${outfile}"
		echo "setenv $name $value" >> ${outfile}
	fi
}

# do_setenv
# @uboot_env	U-boot script in plain text format
# @mtd		MTD device used by U-Boot env.
# @flag		active flag
#
# Save uboot environment variables from u-boot plain text script
#
# Usage:
#     do_setenv /mnt/mmcblk0p1/env/uboot_env_sf /dev/mtd3 1
do_setenv ()
{
	local infile=$1
	local mtd=$2
	local flag=$3

	local crc32=${src_dir}/bin/crc32
	local outfile=/tmp/uboot_env.raw
	local tmpfile=/tmp/uboot_env.tmp

	echo "[do_setenv] infile=$infile"
	echo "[do_setenv] outfile=$outfile"
	echo "[do_setenv] tmpfile=$tmpfile"

	dd if=/dev/zero of=${outfile} bs=1k count=64

	dos2unix -u ${infile}

	cat ${infile} | \
	tr '\n' '~' | sed 's/\\~//g' | tr '~' '\n' | \
	sed 's/\ \+/ /g' | \
	grep setenv | sed 's/setenv //' | sed 's/ /=/' | \
	sed 's/\\;/;/g' | tr "\n" "\0" > ${tmpfile}

	dd if=${tmpfile} of=${outfile} bs=1 seek=5 count=65531 conv=notrunc

	$crc32 -s 5 -x 1 ${outfile}
	$crc32 -s 5 -r ${outfile} > ${tmpfile}
	printf "\x${flag}" >> ${tmpfile}
	dd if=${tmpfile} of=${outfile} bs=1 seek=0 count=5 conv=notrunc

	flash_eraseall $mtd
	cat ${outfile} > $mtd

	#mv ${tmpfile} ${tmpfile}.${flag}
	#cp ${outfile} ${outfile}.${flag}
	rm ${tmpfile} ${outfile}
}



#
# Add additional uboot environment variables to in-file (text script file)
#
# Usage:
#     env_addon outfile
env_addon()
{         
	echo "[WMT] add additionl uboot env..."
	local tmpfile=$1

	local mtd_wload=`get_mtd "w-load-SF"`
	local wload_updateid=`strings ${mtd_wload}  | grep "UPDATEID_"` 
	
	if [ -z "$wload_updateid" ]; then
		echo "[WMT] old w-load in board, use default memtotal."
	else
		local ddr_size=`echo ${wload_updateid} | cut -d_ -f7`
		echo "[WMT] wload_updateid: $wload_updateid, ddr_size = $ddr_size"
		if [ "$ddr_size" == "512MB" ]; then
			echo "[WMT] set memtotal for 512DDR."
			awk 'BEGIN{found = 0;}/setenv memtotal/{found=1}//{if(found == 0){print $0}else{printf("setenv memtotal 448M\n"); found=0;}}' ${tmpfile} > ${tmpfile}.tmp && mv ${tmpfile}.tmp ${tmpfile}
			awk 'BEGIN{found = 0;}/setenv wmt.kernel.animation.addr/{found=1}//{if(found == 0){print $0}else{printf("setenv wmt.kernel.animation.addr 1FC00000\n"); found=0;}}' ${tmpfile} > ${tmpfile}.tmp && mv ${tmpfile}.tmp ${tmpfile}
		elif [ "$ddr_size" == "256MB" ]; then
			echo "[WMT] set memtotal for 256DDR."
			awk 'BEGIN{found = 0;}/setenv memtotal/{found=1}//{if(found == 0){print $0}else{printf("setenv memtotal 201M\n"); found=0;}}' ${tmpfile} > ${tmpfile}.tmp && mv ${tmpfile}.tmp ${tmpfile}
			awk 'BEGIN{found = 0;}/setenv wmt.kernel.animation.addr/{found=1}//{if(found == 0){print $0}else{printf("setenv wmt.kernel.animation.addr FC00000\n"); found=0;}}' ${tmpfile} > ${tmpfile}.tmp && mv ${tmpfile}.tmp ${tmpfile}
		elif [ "$ddr_size" == "1024MB" ]; then
			echo "[WMT] set memtotal for 1024DDR."
			awk 'BEGIN{found = 0;}/setenv memtotal/{found=1}//{if(found == 0){print $0}else{printf("setenv memtotal 864M\n"); found=0;}}' ${tmpfile} > ${tmpfile}.tmp && mv ${tmpfile}.tmp ${tmpfile}
			awk 'BEGIN{found = 0;}/setenv wmt.kernel.animation.addr/{found=1}//{if(found == 0){print $0}else{printf("setenv wmt.kernel.animation.addr 3FC00000\n"); found=0;}}' ${tmpfile} > ${tmpfile}.tmp && mv ${tmpfile}.tmp ${tmpfile}
		else
			echo "[WMT] Error format in wload, use default memtotal."
		fi
	fi

	if [ $ROOT_DEV -eq $DEV_NAND ]; then
		echo "setenv kernel-NAND_mtd `get_mtdblock ${blk_Nand_Kernel}`" >> ${tmpfile}	
		echo "setenv kernel-NAND_ofs `get_mtd_offset ${blk_Nand_Kernel}`" >> ${tmpfile}		
		echo "setenv kernel-NAND_len `get_mtd_len ${blk_Nand_Kernel}`" >> ${tmpfile}
	
		echo "setenv initrd-NAND_mtd `get_mtdblock ${blk_Nand_Initrd}`" >> ${tmpfile}
		echo "setenv initrd-NAND_ofs `get_mtd_offset ${blk_Nand_Initrd}`" >> ${tmpfile}
		echo "setenv initrd-NAND_len `get_mtd_len ${blk_Nand_Initrd}`" >> ${tmpfile}
	
		echo "setenv filesystem-NAND_mtd `get_mtdblock ${blk_Nand_FS}`" >> ${tmpfile}
		echo "setenv filesystem-NAND_ofs `get_mtd_offset ${blk_Nand_FS}`" >> ${tmpfile}
		echo "setenv filesystem-NAND_len `get_mtd_len ${blk_Nand_FS}`" >> ${tmpfile}
	
		local uboot_logo_ofs=`get_mtd_offset ${blk_Nand_Logo}`
		local temp=$(( $uboot_logo_ofs + $UBOOT_LOG2_OFFSET )) 
		local uboot_logo2_ofs=`printf "0x%x" $temp`
		local temp2=$(( $uboot_logo_ofs + $UBOOT_CHARGE_LOG_OFFSET )) 
		local uboot_logo_charge_anim_ofs=`printf "0x%x" $temp2`
		local temp3=$(( $uboot_logo_ofs + $UBOOT_KERNEL_LOG_OFFSET )) 
		local kernel_logo_ofs=`printf "0x%x" $temp3`
	
		echo "setenv wmt.nfc.mtd.u-boot-logo `printf "0x%x" $uboot_logo_ofs`" >> ${tmpfile}
		echo "setenv wmt.nfc.mtd.u-boot-logo2 `printf "0x%x" $uboot_logo2_ofs`" >> ${tmpfile}
		echo "setenv wmt.nfc.mtd.u-boot-chganim `printf "0x%x" $uboot_logo_charge_anim_ofs`" >> ${tmpfile}
		echo "setenv wmt.nfc.mtd.kernel-logo `printf "0x%x" $kernel_logo_ofs`" >> ${tmpfile}
	elif [ $ROOT_DEV -eq $DEV_TF ]; then
		#replace env "kernelargs" from Nand to TF
		sed 's/root=${filesystem-NAND_mtd} noinitrd rootfstype=yaffs2/root=\/dev\/mmcblk1p2 noinitrd rootfstype=ext3/g' < ${tmpfile}  > ${tmpfile}.new && mv ${tmpfile}.new ${tmpfile}
		#sed 's/init=\/init/init=\/init  rootdelay=6/g' < ${tmpfile}  > ${tmpfile}.new && mv ${tmpfile}.new ${tmpfile}
		#replace env "bootcmd" from Nand to TF
		sed 's/nandrw r ${kernel-NAND_ofs} 0x1000000 ${kernel-NAND_len}/fatload mmc 1 0x1000000 uzimage.bin/g' < ${tmpfile}  > ${tmpfile}.new && mv ${tmpfile}.new ${tmpfile}
		#replace env "logocmd" from Nand to TF
		sed 's/nandrw r ${wmt.nfc.mtd.u-boot-logo} ${wmt.display.logoaddr} 10000/mmcinit 1; fatload mmc 1 ${wmt.display.logoaddr} u-boot-logo.data/g' < ${tmpfile}  > ${tmpfile}.new && mv ${tmpfile}.new ${tmpfile}
		sed 's/nandrw r ${wmt.nfc.mtd.u-boot-logo2} ${wmt.display.logoaddr2} 40000/fatload mmc 1 ${wmt.display.logoaddr2} u-boot-logo-ver.data/g' < ${tmpfile}  > ${tmpfile}.new && mv ${tmpfile}.new ${tmpfile}

		#just for logocmd after from Nand to TF
		echo "setenv wmt.nfc.mtd.u-boot-logo 1" >> ${tmpfile}

		echo "setenv rootdev TF" >> ${tmpfile}
	elif [ $ROOT_DEV -eq $DEV_UDISK ]; then
		#replace env "kernelargs" from Nand to UDisk
		sed 's/root=${filesystem-NAND_mtd} noinitrd rootfstype=yaffs2/root=\/dev\/sda2 noinitrd rootfstype=ext3/g' < ${tmpfile}  > ${tmpfile}.new && mv ${tmpfile}.new ${tmpfile}
		#sed 's/init=\/init/init=\/init  rootdelay=6/g' < ${tmpfile}  > ${tmpfile}.new && mv ${tmpfile}.new ${tmpfile}
		#replace env "bootcmd" from Nand to UDisk
		sed 's/nandrw r ${kernel-NAND_ofs} 0x1000000 ${kernel-NAND_len}/fatload usb 0 0x1000000 uzimage.bin/g' < ${tmpfile}  > ${tmpfile}.new && mv ${tmpfile}.new ${tmpfile}
		#replace env "logocmd" from Nand to UDisk
		sed 's/nandrw r ${wmt.nfc.mtd.u-boot-logo} ${wmt.display.logoaddr} 10000/usb reset; fatload usb 0 ${wmt.display.logoaddr} u-boot-logo.data/g' < ${tmpfile}  > ${tmpfile}.new && mv ${tmpfile}.new ${tmpfile}
		sed 's/nandrw r ${wmt.nfc.mtd.u-boot-logo2} ${wmt.display.logoaddr2} 40000/fatload usb 0 ${wmt.display.logoaddr2} u-boot-logo-ver.data/g' < ${tmpfile}  > ${tmpfile}.new && mv ${tmpfile}.new ${tmpfile}
		sed 's/nandrw r ${wmt.nfc.mtd.kernel-logo} ${wmt.kernel.animation.addr} 100000/fatload usb 0 ${wmt.kernel.animation.addr} kernel-logo.data/g' < ${tmpfile}  > ${tmpfile}.new && mv ${tmpfile}.new ${tmpfile}

		#just for logocmd after from Nand to UDisk
		echo "setenv wmt.nfc.mtd.u-boot-logo 1" >> ${tmpfile}

		echo "setenv uboot_usb 8" >> ${tmpfile}
		awk 'BEGIN{found = 0;globalFound=0;}/setenv wmt.usb.param/{found=1}//{if(found == 0){print $0}else{printf("setenv wmt.usb.param 1:3\n"); found=0;globalFound=1;}}END{if(globalFound==0){printf("\nsetenv wmt.usb.param 1:3\n");}}' ${tmpfile} > ${tmpfile}.tmp && mv ${tmpfile}.tmp ${tmpfile}
		echo "setenv rootdev UDisk" >> ${tmpfile}
	else
		echo "[WMT] *E* unkown root device"
	fi

	random_mac ${tmpfile}
	
	restore_env serialnum ${tmpfile}
	
	#cat ${tmpfile}
}

# setenv
# @uboot_env	U-boot script in plain text format
#
# Save uboot environment variables from u-boot plain text script
#
# Usage:
#     setenv "uboot_env_sf"
setenv ()
{
	local mtd_name=
	local mtd=
	local uboot_env=$1
	local tmpfile=/tmp/setenv.tmp

	rm -f ${tmpfile}
	if [ -e ${src_dir}/env/env_uboot ]; then
		cat ${src_dir}/env/env_uboot >> ${tmpfile}
	fi
	if [ -e ${src_dir}/env/${uboot_env} ]; then
		cat ${src_dir}/env/${uboot_env} >> ${tmpfile}
	fi
	if [ -e ${src_dir}/env/uboot_env_tail ]; then
		cat ${src_dir}/env/uboot_env_tail >> ${tmpfile}
	fi
	
	env_addon ${tmpfile}

	mtd_name="u-boot env. cfg. 1-SF"
	mtd=`get_mtd "${mtd_name}"`

	if [ ! -z "${mtd}" ]; then
		do_setenv ${tmpfile} ${mtd} 1
	else
		echo "No MTD partition called ${mtd_name}."
	fi

	mtd_name="u-boot env. cfg. 2-SF"
	mtd=`get_mtd "${mtd_name}"`

	if [ ! -z "${mtd}" ]; then
		do_setenv ${tmpfile} ${mtd} 0
	else
		echo "No MTD partition called ${mtd_name}."
	fi

	#JodyFu: useless for u-boot-env in NAND now
	#setenv_nand ${uboot_env}

	# JodyFu: move next env-var from .config to env/uboot_env_head
	# Use configs from .config or default
	#setenv_if_valid "memtotal" "$MEM_TOTAL"
	#setenv_if_valid "mmcid" "$MMC_ID"
	#setenv_if_valid "lpj" "$LPJ"

	rm -f ${tmpfile}
}

# setenv_if_valid
# @name
# @value
setenv_if_valid ()
{
	if [ $# -lt 2 ]; then
		return
	fi

	if [ ! -z "$2" ] && [ "$2" != "AUTO" ]; then
		setenv.sh "$1" "$2"
	fi
}

# strsearch
# @file
# @string
# @offset
#
# Search a word at the offset of the specific string.
strsearch ()
{
	if [ $# -lt 3 ]; then
		return
	fi

	local match=0
	local ofs=$3
	local uniq_word=`echo $2 | sed 's/ /_/g'`
	local tokens=`strings $1 | sed "s/$2/$uniq_word/g" | tr -d ![]`
	local keyword=`strings $1 | grep "$2"`

	if [ "$keyword" == "" ]; then
		return
	fi

	for i in $tokens ; do
		if [ "$i" == "$uniq_word" ]; then
			match=1
		fi
		if [ $match == 1 ]; then
			if [ $ofs == 0 ]; then
				echo $i
				return
			else
				let "ofs = ofs - 1"
			fi
		fi
	done
}

#
# PLEASE DON'T MODIFY FUNCTIONS ABOVE THIS LINE.
# THOSE FUNCTIONS ARE THE HEART OF BSPINST2.
#

inst_wload_sf ()
{
	local file_wload="w-load.bin"
	local wload_full_path=
	local buildno_board=

	local mtd_wload=`get_mtd "w-load-SF"`
	local wload_updateid=`strings ${mtd_wload}  | grep "UPDATEID_"` 
	
	if [ -z "$wload_updateid" ]; then
		echo "[WMT] old w-load in board, need to update, use default w-load."
		wload_full_path=`find $src_dir -name $file_wload`
	else
		echo "[WMT] wload_updateid: $wload_updateid"
		local wild_wload_name=
		if [ "$wload_updateid" == "UPDATEID_DDR3_800M_800M_32bit_4_512MB" ]; then
			wild_wload_name="*_DDR3_5_3_2_4_512MB"
		elif [ "$wload_updateid" == "UPDATEID_DDR3_800M_800M_16bit_2_256MB" ]; then
			wild_wload_name="*_DDR3_5_3_1_2_256MB"
		else
			wild_wload_name=`echo $wload_updateid | sed 's/UPDATEID/*/g'`
		fi
		
		wild_wload_name=`echo ${wild_wload_name}*`
		echo "[WMT] wild_wload_name: ${wild_wload_name}"
		wload_full_path=`find $src_dir -name ${wild_wload_name}`
		if [ -z "$wload_full_path" ]; then
			echo "[WMT] *W* $wild_wload_name not exist, skip w-load update! "
			return 0
		else
			file_wload=`basename "${wload_full_path}"`
		fi

		buildno_board=`strings ${mtd_wload}  | grep "BUILDID_"` 
	fi

	local buildno_file=`strings ${wload_full_path}  | grep "BUILDID_"` 
	if [ -z "$buildno_file" ]; then
		echo "[WMT] no new w-load, skip w-load update!"
		return 0
	fi
	
	echo "[WMT] buildno_file = $buildno_file,  buildno_board = $buildno_board"
	if [ "$buildno_file" \> "$buildno_board" ]; then
		echo "[WMT] updating $wload_full_path..."
		sf_inst "w-load-SF" "$file_wload"
	else
		echo "[WMT] no newer w-load file, skip"
	fi
}

inst_uboot_sf ()
{
	local file_uboot="u-boot.bin"
	if [ ! -e "$src_dir/$file_uboot" ]; then
		echo "[WMT] *W* not found uboot file, skip u-boot update!"
		return 0;
	fi
	
	local uboot_buildno_file=`strings $src_dir/$file_uboot  | grep "BUILDID_"` 
	if [ -z "$uboot_buildno_file" ] ; then
		echo "[WMT] *I* no new u-boot, skip u-boot update!"
		return 0
	fi

	local mtd_uboot=`get_mtd "u-boot-SF"`
	local uboot_buildno_board=`strings ${mtd_uboot}  | grep "BUILDID_"` 

	echo "[WMT] uboot_buildno_board: $uboot_buildno_board"
	echo "[WMT] uboot_buildno_file: $uboot_buildno_file"

	if [ "$uboot_buildno_file" \> "$uboot_buildno_board" ]; then
		echo "[WMT] updating $file_uboot..."
		sf_inst "u-boot-SF" "$file_uboot"
	else
		echo "[WMT] no newer u-boot file, skip."
	fi
}

inst_kernel_sf ()
{
	sf_inst "kernel-SF" "uzImage.bin"
}

inst_initrd_sf ()
{
	sf_inst "filesystem-SF" "initrd.gz"
}

inst_filesystem_sf ()
{
	sf_inst "filesystem-SF" "ramdisk*.gz"
}

inst_filesystem_cramfs ()
{
       sf_inst "filesystem-SF" "rootfs*.cramfs"
}

inst_wload_nand ()
{
	nand_inst "w-load-NAND" "w-load.bin"
}

inst_uboot_nand ()
{
	nand_inst "u-boot-NAND" "u-boot-nand.bin"
}

inst_kernel_nand ()
{
	nand_inst "${blk_Nand_Kernel}" "uzImage.bin"
}

inst_initrd_nand ()
{
	nand_inst "${blk_Nand_Initrd}" "initrd.gz"
}

isblack ()
{
	local blacklist=
	local res=
	local p=$1

	if [ ! -e $src_dir/blacklist.txt ]; then
		return 0
	fi

	blacklist=`cat $src_dir/blacklist.txt | grep -v \#`
	for b in $blacklist; do
		res=`find $p -name "$b"`
		if [ "$res" != "" ]; then
			return 1
		fi
	done
	return 0
}

isListed ()
{
	local list=
	local res=
	local f=$1
	local p=$2

	if [ -z "$f" ]; then
		return 0
	fi

	if [ ! -e "$src_dir/packages/$f" ]; then
		return 0
	fi

	list=`cat $src_dir/packages/$f | grep -v \#`
	for b in $list; do
		res=`find $p -name "$b"`
		if [ "$res" != "" ]; then
			return 1
		fi
	done
	return 0
}


update_logo ()
{
	local name=$1
	local mtd=
	
	mtd=`get_mtd "$name"`

	if [ -z "$mtd" ]; then
		echo "No MTD partition called $name."
		return 1
	fi

	if [ ! -e "$mtd" ]; then
		return 2
	fi
	
	nand_erase $name
	
	echo "[WMT] update logo"
	Quiet nandwrite -p $mtd $src_dir/u-boot-logo-ver.data
	echo "[WMT] update uboot-logo2"
	Quiet nandwrite -p -s $UBOOT_LOG2_OFFSET  $mtd $src_dir/u-boot-logo.data
	echo "[WMT] update charge-animation in uboot"
	Quiet nandwrite -p -s $UBOOT_CHARGE_LOG_OFFSET  $mtd $src_dir/charge_anim.data
	echo "[WMT] update kernel-logo"
	Quiet nandwrite -p -s $UBOOT_KERNEL_LOG_OFFSET  $mtd $src_dir/kernel-logo.data

	echo "done"
	
	return 0
}


# inst_filesystem
#
# Install software packages into NAND rootfs
#
# The default search path is the current folder
# if $src_dir is invalid.
inst_filesystem ()
{
	local sys_mnt=$mnt_fs
	local data_mnt=$mnt_data
	local mtd_filesystem=
	local mtd_data=
	local prev_ratio=$CompleteRatio
	local _ratio=
	local ret=

	mkdir -p ${sys_mnt}

	if [ $ROOT_DEV -eq $DEV_NAND ]; then
		mtd_filesystem=`get_mtdblock "${blk_Nand_FS}"`
		mtd_data=`get_mtdblock "${blk_Nand_AndroidData}"`

		msg "Erasing System partition(Nand)..."
		umount ${mtd_filesystem}
		nand_erase ${blk_Nand_FS}
		
		msg "Erasing Data partition(Nand)..."
		umount ${mtd_data}
		nand_erase "${blk_Nand_AndroidData}"
	
		mount -t yaffs2 -o rw $mtd_filesystem ${sys_mnt}
		mkdir -p ${data_mnt}
		mount -t yaffs2 -o rw $mtd_data ${data_mnt}
	elif [ $ROOT_DEV -eq $DEV_TF ] || [ $ROOT_DEV -eq $DEV_UDISK ]; then
		if [ $ROOT_DEV -eq $DEV_TF ]; then
			msg "Prepare system/data partition(TF)..."
			mtd_filesystem=/dev/mmcblk1p2
			mtd_data=/dev/mmcblk1p5
		elif [ $ROOT_DEV -eq $DEV_UDISK ]; then
			msg "Prepare system/data partition(UDISK)..."
			mtd_filesystem=/dev/sda2
			mtd_data=/dev/sda5
		fi

		msg "Erasing System partition..."
		umount ${mtd_filesystem}
		mkfs.ext3 ${mtd_filesystem}

		msg "Erasing Data partition..."
		umount ${mtd_data}
		mkfs.ext3 ${mtd_data}
		if [ $? -ne 0 ] ; then
			msg "*E*  Failed to format FileSystem partition(UDISK)..."
			exit 0
		fi
		
		mount ${mtd_filesystem} ${sys_mnt}
		mkdir -p ${data_mnt}
		mount ${mtd_data} ${data_mnt}
	else
		msg "*E* Unknown root device!"
		return 1
	fi
	
	if [ $? -ne 0 ] ; then
		msg "*E* FileSystem partition not found!"
		return 1
	fi
	
	local_cwd="$src_dir/packages"
	if [ -e "${local_cwd}" ]; then
		msg "Installing system partition..."
		packages=`find ${local_cwd} -maxdepth 2 -name "*.tgz" | sort`
		pkgtotal=`find ${local_cwd} -maxdepth 2 -name "*.tgz" | wc -l`
		pkgcount=0
		echo
		for f in $packages ; do
			let 'pkgcount = pkgcount + 1'
			_ratio=`expr $prev_ratio + \( 70 - $prev_ratio \) \* $pkgcount / $pkgtotal`
			ratio $_ratio
			msg "    `base_name $local_cwd/ $f`"
			echo "Install $f ($pkgcount / $pkgtotal)"
			tar zxf $f -C ${sys_mnt} >/dev/null 
			sleep 1
		done
	fi

#   extract patch packages to Android data partition
	msg "Installing data partition..."
	/bin/tar xf $src_dir/packages/fs_patch/.restore/data.tar -C  ${data_mnt} >/dev/null 
	/bin/tar xf $src_dir/packages/fs_patch/.restore/data_app.tar -C  ${data_mnt} >/dev/null 
	ratio 85

	msg "Copy fs_patch&restore files..."
	/bin/cp -aR $src_dir/packages/fs_patch/. ${sys_mnt}
	echo "[WMT] copy all data app for restore"
	/bin/cp -aR $src_dir/packages/fs_patch/data/app ${sys_mnt}/.restore/data_app

#   remove customer key information
	/bin/rm -rf ${sys_mnt}/.restore/wmtpref
	
	/bin/chmod -R 777  ${data_mnt}/app

#   no need to chmod/chown if the patch is dumped from Anroid runtime system
#   normally, you can't modify this dumped patch
#   you must use sudo to modify and re-compress the pactch if want to modify it
#	/bin/chmod 0660  ${data_mnt}/data/com.android.providers.settings/databases/*
#   1000 is uid:guid of settings.apk's
#	/bin/chown -hR 1000:1000  ${data_mnt}/data/com.android.providers.settings
	#add super mode and change owner for 'su' (it will be called by SuperUser.apk)
	/bin/chown 0:0  ${sys_mnt}/system/xbin/su 
	/bin/chmod 4755  ${sys_mnt}/system/xbin/su 

	if [ $ROOT_DEV -eq $DEV_TF ]; then
		#replace data&cache partition from Nand to TF
		sed 's/mount yaffs2 mtd@userdata \/data nosuid nodev/mount ext3 \/dev\/block\/mmcblk1p5 \/data/g' < ${sys_mnt}/init.rc  > ${sys_mnt}/init.rc.new && mv ${sys_mnt}/init.rc.new ${sys_mnt}/init.rc
		sed 's/mount yaffs2 mtd@cache \/cache nosuid nodev/mount ext3 \/dev\/block\/mmcblk1p6 \/cache/g' < ${sys_mnt}/init.rc  > ${sys_mnt}/init.rc.new && mv ${sys_mnt}/init.rc.new ${sys_mnt}/init.rc
	elif [ $ROOT_DEV -eq $DEV_UDISK ]; then
		#replace data&cache partition from Nand to UDISK
		sed 's/mount yaffs2 mtd@userdata \/data nosuid nodev/mount ext3 \/dev\/block\/sda5 \/data/g' < ${sys_mnt}/init.rc  > ${sys_mnt}/init.rc.new && mv ${sys_mnt}/init.rc.new ${sys_mnt}/init.rc
		sed 's/mount yaffs2 mtd@cache \/cache nosuid nodev/mount ext3 \/dev\/block\/sda6 \/cache/g' < ${sys_mnt}/init.rc  > ${sys_mnt}/init.rc.new && mv ${sys_mnt}/init.rc.new ${sys_mnt}/init.rc
	fi
	sync
	
	return 0
}

reboot_countdown ()
{
	for t in 3 2 1 0 ; do
		msg "Don't remove installation disk! ($t)"
		sleep 1
	done
	msg "Rebooting for installation stage 2"

	sleep 5
	reboot -f -n
}

# Install senarios
# 0. SF 16MiB              (ramdisk)"
# 1. SF 16MiB  + NAND      (yaffs2)"
# 2. SF/NAND 512KiB + NAND (yaffs2)"
# 3. SD/MMC                (ramdisk)"
# 4. SF 8MiB               (ramdisk)"
# 5. SF 8,16MiB            (cramfs)"

# pre_install
#
# Installation phase 1
#
pre_install ()
{
	echo "Executing pre_install($1)"

	case $1 in
	0)
		msg "Upgrading W-Load to SF."
		inst_wload_sf
		ratio 10

		msg "Upgrading U-Boot to SF."
		inst_uboot_sf
		ratio 20

		msg "Upgrading U-Boot env. cfg. to SF."
		setenv "uboot_env_sf"
		ratio 30

		msg "Upgrading Kernel to SF."
		inst_kernel_sf
		ratio 40

		msg "Upgrading Ramdisk to SF."
		inst_filesystem_sf
		ratio 100
		;;
	1)
		msg "Upgrading W-Load to SF."
		inst_wload_sf
		ratio 10

		msg "Upgrading U-Boot to SF."
		inst_uboot_sf
		ratio 20

		msg "Upgrading U-Boot env. cfg. to SF."
		setenv "uboot_env_sf"
		ratio 30

		msg "Upgrading Kernel to SF."
		inst_kernel_sf
		ratio 40

		msg "Upgrading Ramdisk to SF."
		inst_initrd_sf
		ratio 50

		reboot_countdown
		;;
	2)
		msg "Upgrading W-Load to SF."
		inst_wload_sf
		ratio 5

		msg "Upgrading W-Load to NAND."
		inst_wload_nand
		ratio 10

		msg "Upgrading U-Boot to SF."
		inst_uboot_sf
		ratio 15

		msg "Upgrading U-Boot to SF."
		inst_uboot_nand
		ratio 20

		local initrd=`get_mtd "${blk_Nand_Initrd}"`

		if [ "$initrd" != "" ] ; then
			msg "Upgrading U-Boot env. cfg. to SF."
			setenv uboot_env_nand_r
			ratio 30

			msg "Upgrading Kernel to NAND."
			inst_kernel_nand
			ratio 40

			msg "Upgrading Ramdisk to NAND."
			inst_initrd_nand
			ratio 50

			reboot_countdown
		else
			msg "Cannot upgrade initrd-NAND!!!"
			sleep 10

			#msg "Upgrading U-Boot env. cfg. to SF."
			#setenv uboot_env_mmc
			#ratio 30
		fi
		;;
	3)
		msg "Upgrading U-Boot env. cfg. to SF."
		setenv uboot_env_mmc
		ratio 100
		;;
	4)
		msg "Upgrading W-Load to SF."
		inst_wload_sf
		ratio 10

		msg "Upgrading U-Boot to SF."
		inst_uboot_sf
		ratio 20

		msg "Upgrading U-Boot env. cfg. to SF."
		setenv "uboot_env_sf_8mb"
		ratio 30

		msg "Upgrading Kernel to SF."
		inst_kernel_sf
		ratio 40

		msg "Upgrading Ramdisk to SF."
		inst_filesystem_sf
		ratio 100
		;;
	5)
		msg "Upgrading W-Load to SF."
		inst_wload_sf
		ratio 10

		msg "Upgrading U-Boot to SF."
		inst_uboot_sf
		ratio 20

		msg "Upgrading U-Boot env. cfg. to SF."
		setenv "uboot_env_sf_cramfs"
		ratio 30

		msg "Upgrading Kernel to SF."
		inst_kernel_sf
		ratio 40

		msg "Upgrading Cramfs to SF."
		inst_filesystem_cramfs
		ratio 100
		;;
	esac
}

# do_install
#
# Installation phase 2
do_install ()
{
	echo "Executing do_install($1)"

	case $1 in
	0)
		msg "Upgrading W-Load to SF."
		inst_wload_sf
		ratio 10

		msg "Upgrading U-Boot to SF."
		inst_uboot_sf
		ratio 20

		msg "Upgrading U-Boot env. cfg. to SF."
		setenv "uboot_env_sf"
		ratio 30

		msg "Upgrading Kernel to SF."
		inst_kernel_sf
		ratio 40

		msg "Upgrading Ramdisk to SF."
		inst_filesystem_sf
		ratio 100
		;;
	1)
		msg "Upgrading W-Load to SF."
		inst_wload_sf
		ratio 10

		msg "Upgrading U-Boot to SF."
		inst_uboot_sf
		ratio 20

		msg "Upgrading U-Boot env. cfg. to SF."
		setenv "uboot_env_sf_nand"
		ratio 30

		msg "Upgrading Kernel to SF."
		inst_kernel_sf
		ratio 40

		msg "Upgrading filesystem to NAND."
		inst_filesystem
		ratio 100
		;;
	2)
		echo "[DBG] ROOT_DEV=$ROOT_DEV, DEV_TF=$DEV_TF, DEV_UDISK=$DEV_UDISK"
		if [ $ROOT_DEV -eq $DEV_TF ]; then
			if [ -e "$src_dir/bin/partition.sh" ]; then
				WMT_Fdisk "$src_dir/bin/partition.sh" "/dev/mmcblk1"
			else
				msg "Partition file not found, upgrading stopped!"
				exit 0
			fi
		elif [ $ROOT_DEV -eq $DEV_UDISK ]; then
			if [ -e "$src_dir/bin/partition.sh" ]; then
				WMT_Fdisk "$src_dir/bin/partition.sh" "/dev/sda"
			else
				msg "Partition file not found, upgrading stopped!"
				exit 0
			fi
		fi

		msg "Upgrading W-Load to SF."
		inst_wload_sf
		ratio 5

		#JodyFu: useless for w-load in NAND now
		#msg "Upgrading W-Load to NAND."
		#inst_wload_nand
		#ratio 10

		msg "Upgrading U-Boot to SF."
		inst_uboot_sf
		ratio 15

		#JodyFu: useless for u-boot in NAND now
		#msg "Upgrading U-Boot to NAND."
		#inst_uboot_nand
		#ratio 20

		msg "Upgrading U-Boot env. cfg. to SF."
		setenv "uboot_env_nand"
		ratio 25

		if [ $ROOT_DEV -eq $DEV_NAND ]; then
			local mtd=`get_mtd "${blk_Nand_Kernel}"`
			if [ "$mtd" == "" ] ; then
				msg "Cannot upgrade kernel-NAND!!!"
				return 1
			fi

			#JodyFu: just for factory recovery
			msg "Upgrading Ramdisk to NAND."
			inst_initrd_nand
			ratio 30

			msg "Upgrading Kernel to NAND."
			inst_kernel_nand
			ratio 33

			msg "Upgrading U-Boot LOGO to NAND"
			update_logo "${blk_Nand_Logo}"
			ratio 38
			
		elif [ $ROOT_DEV -eq $DEV_TF ] || [ $ROOT_DEV -eq $DEV_UDISK ]; then
			local mtd_kernel=
			local mtd_all=
			if [ $ROOT_DEV -eq $DEV_TF ]; then
				msg "Prepare kernel partition(TF)..."
				mtd_kernel=/dev/mmcblk1p1
				mtd_all=/dev/mmcblk1p*
			elif [ $ROOT_DEV -eq $DEV_UDISK ]; then
				msg "Prepare kernel partition(UDISK)..."
				mtd_kernel=/dev/sda1
				mtd_all=/dev/sda*
			fi

			mkdosfs ${mtd_kernel}
			if [ $? -ne 0 ] ; then
				echo "[WMT] *W* umount all partition and try again!"
				umount ${mtd_all}
				mkdosfs ${mtd_kernel}
				if [ $? -ne 0 ] ; then
					msg "*E*  Failed to format kernel partition!!"
					exit 0
				fi
			fi

			/bin/mkdir -p ${mnt_kernel}
			mount ${mtd_kernel} $mnt_kernel
			if [ $? -ne 0 ] ; then
				msg "*E*  Failed to mount kernel partition..."
			    exit 0
			fi
			msg "Upgrading Kernel/Ramdisk/Logo ..."
			/bin/cp -af $src_dir/uzImage.bin $mnt_kernel
			/bin/cp -af $src_dir/initrd.gz $mnt_kernel
			/bin/cp -af $src_dir/u-boot-logo.data $mnt_kernel
			/bin/cp -af $src_dir/u-boot-logo-ver.data $mnt_kernel
			/bin/cp -af $src_dir/kernel-logo.data $mnt_kernel			
			/bin/cp -af $src_dir/charge_anim.data $mnt_kernel			
			ratio 30
		else
			msg "*E* Unknown root device!"
			return 3
		fi

		inst_filesystem
		ratio 90
		;;
	3)
		msg "Upgrading U-Boot env. cfg. to SF."
		setenv uboot_env_mmc
		ratio 100
		;;
	4)
		msg "Upgrading W-Load to SF."
		inst_wload_sf
		ratio 10

		msg "Upgrading U-Boot to SF."
		inst_uboot_sf
		ratio 20

		msg "Upgrading U-Boot env. cfg. to SF."
		setenv "uboot_env_sf_8mb"
		ratio 30

		msg "Upgrading Kernel to SF."
		inst_kernel_sf
		ratio 40

		msg "Upgrading Ramdisk to SF."
		inst_filesystem_sf
		ratio 100
		;;
	5)
		msg "Upgrading W-Load to SF."
		inst_wload_sf
		ratio 10

		msg "Upgrading U-Boot to SF."
		inst_uboot_sf
		ratio 20

		msg "Upgrading U-Boot env. cfg. to SF."
		setenv "uboot_env_sf_cramfs"
		ratio 30

		msg "Upgrading Kernel to SF."
		inst_kernel_sf
		ratio 40

		msg "Upgrading Cramfs to SF."
		inst_filesystem_cramfs
		ratio 100
		;;
	esac
	
	return 0
}

# Get key from IR
GetKeyCode ()
{
	local f=/usr/bin/input.sh
	local k=/tmp/keycode
	if [ ! -e $f ]; then
		echo "#!/bin/sh" > $f
		echo "echo \$1 > $k" >> $f
		chmod 755 $f
		sync
	fi

	if [ -e $k ]; then
		cat $k
		> $k
	fi
}

show_next_mac ()
{
	let "s = 0"
	for i in 1 2 3 4 5 6; do
		let "u$i = `echo $1|cut -d: -f$i|xargs printf "0x%s"|\
			xargs printf "%u"`"
		let "s = s * 256 + u$i"
	done

	let "s = s + 1"

	for i in 6 5 4 3 2 1; do
		let "u$i = `expr $s % 256`"
		let "s = s / 256 "
	done
	
	printf "%02x:%02x:%02x:%02x:%02x:%02x\n" $u1 $u2 $u3 $u4 $u5 $u6 |\
		tr [a-z] [A-Z]
}

edit_config ()
{
	if [ $# -lt 2 ]; then
		echo "Usage: edit_config config name [value]"
		return
	fi

	local config=$1
	local name=$2
	local value=$3
	local old_value=

	if [ -e "$config" ]; then
		old_value=`cat $config | grep $name | cut -d= -f2`
	fi

	if [ -z "$old_value" ]; then
		if [ ! -z "$value" ]; then
			# add
			echo "$name=$value" >> $config
			return
		fi
	else
		if [ ! -z "$value" ]; then
			# modify
			sed "s/$name=$old_value/$name=$value/g" -i $config
		else
			# delete
			sed "s/$name=$old_value//g" -i $config
		fi
	fi
	sed "/^$/d" -i $config
}

save_config ()
{
	local f="$src_dir/.config"
	edit_config $f DEFAULT_SCENARIO $DEFAULT_SCENARIO
	edit_config $f INSTALL_BASIC $INSTALL_BASIC
	edit_config $f INSTALL_RECOMMENDED $INSTALL_RECOMMENDED
	edit_config $f INSTALL_FULL $INSTALL_FULL
	edit_config $f TIMEOUT_MENU $TIMEOUT_MENU
	edit_config $f TIMEOUT_STARTUP $TIMEOUT_STARTUP
	edit_config $f TIMEOUT_EXIT $TIMEOUT_EXIT
	edit_config $f MAC_0 $MAC_0
	edit_config $f MAC_INCREMENT $MAC_INCREMENT
	edit_config $f MEM_TOTAL $MEM_TOTAL
	sync
	cat $f
}

load_config ()
{
	local f="$src_dir/.config"
	if [ -e "$f" ]; then
		source $f
		cat $f
	else
		save_config
	fi
}

netconfig ()
{
	# do nothing, because using random_mac
	return
	
	local f="$src_dir/.config"

	if [ "$MAC_0" == "AUTO" ]; then
		HWADDR_0=`ifconfig|grep eth0|sed 's/HWaddr /@/'|cut -d@ -f2`
	else
		HWADDR_0=$MAC_0
	fi
	SetMessage `FM 4` `FM 11` "MAC_0 = $HWADDR_0"
	setenv.sh "ethaddr" "$HWADDR_0"

	if [ "$MAC_INCREMENT" != "0" ] && [ "$MAC_0" != "AUTO" ]; then
		MAC_0=`show_next_mac $MAC_0`
		edit_config $f MAC_0 "$MAC_0"
		echo "Next MAC_0 will be $MAC_0"
	fi
}

